// SimpleRefDataExample.cpp : Defines the entry point for the console application.
//
#include <blpapi_session.h>
#include <blpapi_eventdispatcher.h>

#include <blpapi_event.h>
#include <blpapi_message.h>
#include <blpapi_element.h>
#include <blpapi_name.h>
#include <blpapi_request.h>
#include <blpapi_subscriptionlist.h>
#include <blpapi_defs.h>
#include <time.h>
#include <sstream>
#include <iomanip>
#include <iostream>
#include <string>
#include <stdlib.h>
#include <string.h>

#include "mathlink.h"
extern void BBRetrieveIntradayBars(const char* Ticker,const char* Field,const char* StartTime, const char* EndTime, int BarSize, int Fill);

using namespace std;
using namespace BloombergLP;
using namespace blpapi;

namespace {
	const Name BAR_DATA("barData");
    const Name BAR_TICK_DATA("barTickData");
    const Name OPEN("open");
    const Name HIGH("high");
    const Name LOW("low");
    const Name CLOSE("close");
    const Name VOLUME("volume");
    const Name NUM_EVENTS("numEvents");
    const Name TIME("time");
    const Name RESPONSE_ERROR("responseError");
    const Name SESSION_TERMINATED("SessionTerminated");
    const Name CATEGORY("category");
    const Name MESSAGE("message");
	const Name SESSION_STARTUP_FAILURE("SessionStartupFailure");
}

class IntradayBars
{
	std::string         d_host;
    int                 d_port;

    void printUsage()
    {
		std::cout << "Usage:" << std::endl
			<< "	Retrieve reference data " << std::endl
			<< "		[-ip 		<ipAddress	= localhost>" << std::endl
			<< "		[-p 		<tcpPort	= 8194>" << std::endl;
		return;
    }

	void printErrorInfo(const char *leadingStr, const Element &errorInfo)
    {
        std::cout << leadingStr
            << errorInfo.getElementAsString(CATEGORY)
            << " ("
            << errorInfo.getElementAsString(MESSAGE)
            << ")" << std::endl;
    }

	string processEvent(Event &event)
    {
        string output = "Failed";
		MessageIterator msgIter(event);
        while (msgIter.next()) {
            Message msg = msgIter.message();
            if (msg.hasElement(RESPONSE_ERROR)) {
                printErrorInfo("REQUEST FAILED: ", 
                    msg.getElement(RESPONSE_ERROR));
                continue;
            }
            output = processMessage(msg);
        }
		return output;
    }

	string processMessage(Message &msg)
    {
		string output;
        Element data = msg.getElement(BAR_DATA).getElement(BAR_TICK_DATA);
        int numBars = data.numValues();
        std::cout <<"Response contains " << numBars << " bars" << std::endl;
        std::cout <<"Datetime\t\tOpen\t\tHigh\t\tLow\t\tClose" <<
            "\t\tNumEvents\tVolume" << std::endl;
		output = "{";
        for (int i = 0; i < numBars; ++i) {
            Element bar = data.getValueAsElement(i);
            Datetime time = bar.getElementAsDatetime(TIME);
			std::string timeString = bar.getElementAsString(TIME);
            double open = bar.getElementAsFloat64(OPEN);
            double high = bar.getElementAsFloat64(HIGH);
            double low = bar.getElementAsFloat64(LOW);
            double close = bar.getElementAsFloat64(CLOSE);
            int numEvents = bar.getElementAsInt32(NUM_EVENTS);
            long long volume = bar.getElementAsInt64(VOLUME);

            std::cout.setf(std::ios::fixed, std::ios::floatfield);
            std::cout << time.month() << '/' << time.day() << '/' << time.year()
                << " " << time.hours() << ":" << time.minutes()
                <<  "\t\t" << std::showpoint
                << std::setprecision(3) << open << "\t\t"
                << high << "\t\t"
                << low <<  "\t\t"
                << close <<  "\t\t"
                << numEvents <<  "\t\t"
                << std::noshowpoint
                << volume << std::endl;

			output += "{\"";
			output += timeString;
			output += "\", ";
			output += bar.getElementAsString(OPEN);
			output += ", ";
			output += bar.getElementAsString(HIGH);
			output += ", ";
			output += bar.getElementAsString(LOW);
			output += ", ";
			output += bar.getElementAsString(CLOSE);
			output += ", ";
			output += bar.getElementAsString(NUM_EVENTS);
			output += ", ";
			output += bar.getElementAsString(VOLUME);
			output += "}";
			if(i < numBars-1)
				output += ",";
        }
		output += "}";
		return output;
    }

    public:
	string Retrieve(const char* Ticker,const char* Field,const char* StartTime, const char* EndTime, int BarSize, int Fill)
    {
        string output = "Failed";
		d_host = "localhost";
        d_port = 8194;

        SessionOptions sessionOptions;
        sessionOptions.setServerHost(d_host.c_str());
        sessionOptions.setServerPort(d_port);

        std::cout << "Connecting to " <<  d_host << ":" 
                  << d_port << std::endl;
        Session session(sessionOptions);
        if (!session.start()) {
            std::cerr <<"Failed to start session." << std::endl;
            return "Failed to open";
        }
        if (!session.openService("//blp/refdata")) {
            std::cerr <<"Failed to open //blp/refdata" << std::endl;
            return "Failed to open";
        }

        Service refDataService = session.getService("//blp/refdata");
        Request request = refDataService.createRequest("IntradayBarRequest");

		// only one security/eventType per request
        request.set("security", Ticker);
		request.set("eventType", Field);
        request.set("interval", BarSize);

        
		
		// Times are in GMT
		request.set("startDateTime", StartTime);
		request.set("endDateTime", EndTime);

		if(Fill)
			request.set("gapFillInitialBar", true);
		else
			request.set("gapFillInitialBar", false);

        std::cout << "Sending Request: " << request << std::endl;
        session.sendRequest(request);

		bool done = false;
        while (!done) {
            Event event = session.nextEvent();
            if (event.eventType() == Event::PARTIAL_RESPONSE) {
                std::cout << "Processing Partial Response" << std::endl;
                output = processEvent(event);
            }
            else if (event.eventType() == Event::RESPONSE) {
                std::cout << "Processing Response" << std::endl;
                output = processEvent(event);
				done = true;
            } else {
                MessageIterator msgIter(event);
                while (msgIter.next()) {
                    Message msg = msgIter.message();
                    if (event.eventType() == Event::SESSION_STATUS) {
                        if (msg.messageType() == SESSION_TERMINATED ||
                            msg.messageType() == SESSION_STARTUP_FAILURE) {
                            done = true;
                        }
                    }
                }
            }
        }
		return output;
	}
};


void BBRetrieveIntradayBars(const char* Ticker,const char* Field,const char* StartTime, const char* EndTime, int BarSize, int Fill)
{
	string temp;
	//Create SimpleHistoryExample Class Object
	IntradayBars Example;
	//Retrieve string created by the example object
	temp = Example.Retrieve(Ticker,Field,StartTime,EndTime,BarSize,Fill);
	std::cout<<"Temp: "<<temp<<std::endl;
	//Put string to Mathematica
	MLPutString(stdlink, temp.c_str());
	return;
}



#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
{
	/* Due to a bug in some standard C libraries that have shipped with
	 * MPW, zero is passed to MLMain below.  (If you build this program
	 * as an MPW tool, you can change the zero to argc.)
	 */
	argc = argc; /* suppress warning */
	return MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
{
	char  buff[512];
	char FAR * buff_start = buff;
	char FAR * argv[32];
	char FAR * FAR * argv_end = argv + 32;

	hinstPrevious = hinstPrevious; /* suppress warning */

	if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
	MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
	return MLMain( (int)(argv_end - argv), argv);
}

#else

int main(int argc, char* argv[])
{
	return MLMain(argc, argv);
}

#endif